@@ -17,20 +17,20 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2007 Red Hat, Inc.
+ * (C) Copyright 2007 - 2011 Red Hat, Inc.
  */
 
+#include <config.h>
 #include <string.h>
 #include <netinet/ether.h>
 #include <glib.h>
-#include <glib/gi18n.h>
 
 #include <nm-device-ethernet.h>
 #include <nm-device-wifi.h>
-#include <nm-gsm-device.h>
-#include <nm-cdma-device.h>
+#include <nm-device-bt.h>
+#include <nm-device-modem.h>
+#include <nm-device-wimax.h>
 #include <nm-access-point.h>
-#include <nm-settings.h>
 
 #include <nm-setting-connection.h>
 #include <nm-setting-wired.h>
@@ -40,10 +40,11 @@
 #include <nm-setting-gsm.h>
 #include <nm-setting-cdma.h>
 #include <nm-setting-pppoe.h>
+#include <nm-setting-bluetooth.h>
+#include <nm-setting-wimax.h>
 #include <nm-utils.h>
 
 #include "utils.h"
-#include "gconf-helpers.h"
 
 static char *ignored_words[] = {
 	"Semiconductor",
@@ -56,6 +57,7 @@ static char *ignored_words[] = {
 	"Co.",
 	"Inc.",
 	"Inc",
+	"Incorporated",
 	"Ltd.",
 	"Limited.",
 	"Intel?",
@@ -63,6 +65,7 @@ static char *ignored_words[] = {
 	"adapter",
 	"[hex]",
 	"NDIS",
+	"Module",
 	NULL
 };
 
@@ -85,6 +88,7 @@ static char *ignored_phrases[] = {
 	"Business Mobile Networks BV",
 	"Mobile Broadband Minicard Composite Device",
 	"Mobile Communications AB",
+	"(PC-Suite Mode)",
 	NULL
 };
 
@@ -171,12 +175,17 @@ utils_get_device_description (NMDevice *device)
 
 	str = g_string_new_len (NULL, strlen (vendor) + strlen (product) + 1);
 
-	g_string_append (str, vendor);
-	g_free (vendor);
+	/* Another quick hack; if all of the fixed up vendor string
+	 * is found in product, ignore the vendor.
+	 */
+	if (!strcasestr (product, vendor)) {
+		g_string_append (str, vendor);
+		g_string_append_c (str, ' ');
+	}
 
-	g_string_append_c (str, ' ');
 	g_string_append (str, product);
 	g_free (product);
+	g_free (vendor);
 
 	description = str->str;
 	g_string_free (str, FALSE);
@@ -188,302 +197,6 @@ utils_get_device_description (NMDevice *device)
 	return description;
 }
 
-static GByteArray *
-file_to_g_byte_array (const char *filename)
-{
-	char *contents = NULL;
-	GByteArray *array = NULL;
-	gsize length = 0;
-
-	if (!g_file_get_contents (filename, &contents, &length, NULL))
-		return NULL;
-
-	array = g_byte_array_sized_new (length);
-	if (!array) {
-		g_free (contents);
-		return NULL;
-	}
-
-	g_byte_array_append (array, (unsigned char *) contents, length);
-	return array;
-}
-
-static gboolean
-fill_one_private_key (NMConnection *connection,
-                      const char *pk_tag,
-                      const char *pk_prop,
-                      const char *cc_prop)
-{
-	const char *filename;
-	NMSetting8021x *tmp;
-	NMSetting8021xCKType pk_type = NM_SETTING_802_1X_CK_TYPE_UNKNOWN;
-	gboolean need_client_cert = TRUE;
-
-	/* If the private key is PKCS#12, don't set the client cert */
-	filename = g_object_get_data (G_OBJECT (connection), pk_tag);
-	if (!filename)
-		return TRUE;
-
-	tmp = NM_SETTING_802_1X (nm_setting_802_1x_new ());
-	nm_setting_802_1x_set_private_key_from_file (tmp, filename, NULL, &pk_type, NULL);
-	if (pk_type == NM_SETTING_802_1X_CK_TYPE_PKCS12) {
-		GByteArray *array;
-
-		array = file_to_g_byte_array (filename);
-		if (array) {
-			NMSetting *s_8021x = nm_connection_get_setting (connection, NM_TYPE_SETTING_802_1X);
-
-			g_object_set (s_8021x,
-			              pk_prop, array,
-			              cc_prop, array,
-			              NULL);
-			g_byte_array_free (array, TRUE);
-			need_client_cert = FALSE;
-		}
-	}
-	g_object_unref (tmp);
-	return need_client_cert;
-}
-
-gboolean
-utils_fill_connection_certs (NMConnection *connection, GError **error)
-{
-	NMSetting8021x *s_8021x;
-	const char *filename;
-	GError *tmp_error = NULL;
-	gboolean need_client_cert = TRUE;
-
-	g_return_val_if_fail (connection != NULL, FALSE);
-
-	s_8021x = NM_SETTING_802_1X (nm_connection_get_setting (connection, NM_TYPE_SETTING_802_1X));
-	if (!s_8021x)
-		return TRUE;
-
-	filename = g_object_get_data (G_OBJECT (connection), NMA_PATH_CA_CERT_TAG);
-	if (filename) {
-		if (!nm_setting_802_1x_set_ca_cert_from_file (s_8021x, filename, NULL, &tmp_error)) {
-			g_set_error (error, tmp_error->domain, tmp_error->code,
-			             _("Could not read CA certificate: %s"), tmp_error->message);
-			g_clear_error (&tmp_error);
-			return FALSE;
-		}
-	}
-
-	/* If the private key is PKCS#12, don't set the client cert */
-	need_client_cert = fill_one_private_key (connection,
-	                                         NMA_PATH_PRIVATE_KEY_TAG,
-	                                         NM_SETTING_802_1X_PRIVATE_KEY,
-	                                         NM_SETTING_802_1X_CLIENT_CERT);
-	if (need_client_cert) {
-		filename = g_object_get_data (G_OBJECT (connection), NMA_PATH_CLIENT_CERT_TAG);
-		if (filename) {
-			if (!nm_setting_802_1x_set_client_cert_from_file (s_8021x, filename, NULL, &tmp_error)) {
-				g_set_error (error, tmp_error->domain, tmp_error->code,
-				             _("Could not read client certificate: %s"), tmp_error->message);
-				g_clear_error (&tmp_error);
-				return FALSE;
-			}
-		}
-	}
-
-	filename = g_object_get_data (G_OBJECT (connection), NMA_PATH_PHASE2_CA_CERT_TAG);
-	if (filename) {
-		if (!nm_setting_802_1x_set_phase2_ca_cert_from_file (s_8021x, filename, NULL, &tmp_error)) {
-			g_set_error (error, tmp_error->domain, tmp_error->code,
-			             _("Could not read inner CA certificate: %s"), tmp_error->message);
-			g_clear_error (&tmp_error);
-			return FALSE;
-		}
-	}
-
-	/* If the private key is PKCS#12, don't set the client cert */
-	need_client_cert = fill_one_private_key (connection,
-	                                         NMA_PATH_PHASE2_PRIVATE_KEY_TAG,
-	                                         NM_SETTING_802_1X_PHASE2_PRIVATE_KEY,
-	                                         NM_SETTING_802_1X_PHASE2_CLIENT_CERT);
-	if (need_client_cert) {
-		filename = g_object_get_data (G_OBJECT (connection), NMA_PATH_PHASE2_CLIENT_CERT_TAG);
-		if (filename) {
-			if (!nm_setting_802_1x_set_phase2_client_cert_from_file (s_8021x, filename, NULL, &tmp_error)) {
-				g_set_error (error, tmp_error->domain, tmp_error->code,
-				             _("Could not read inner client certificate: %s"), tmp_error->message);
-				g_clear_error (&tmp_error);
-				return FALSE;
-			}
-		}
-	}
-
-	return TRUE;
-}
-
-void
-utils_clear_filled_connection_certs (NMConnection *connection)
-{
-	NMSetting8021x *s_8021x;
-
-	g_return_if_fail (connection != NULL);
-
-	s_8021x = NM_SETTING_802_1X (nm_connection_get_setting (connection, NM_TYPE_SETTING_802_1X));
-	if (!s_8021x)
-		return;
-
-	g_object_set (s_8021x,
-	              NM_SETTING_802_1X_CA_CERT, NULL,
-	              NM_SETTING_802_1X_CLIENT_CERT, NULL,
-	              NM_SETTING_802_1X_PRIVATE_KEY, NULL,
-	              NM_SETTING_802_1X_PHASE2_CA_CERT, NULL,
-	              NM_SETTING_802_1X_PHASE2_CLIENT_CERT, NULL,
-	              NM_SETTING_802_1X_PHASE2_PRIVATE_KEY, NULL,
-	              NULL);
-}
-
-
-struct cf_pair {
-	guint32 chan;
-	guint32 freq;
-};
-
-static struct cf_pair a_table[] = {
-	/* A band */
-	{  7, 5035 },
-	{  8, 5040 },
-	{  9, 5045 },
-	{ 11, 5055 },
-	{ 12, 5060 },
-	{ 16, 5080 },
-	{ 34, 5170 },
-	{ 36, 5180 },
-	{ 38, 5190 },
-	{ 40, 5200 },
-	{ 42, 5210 },
-	{ 44, 5220 },
-	{ 46, 5230 },
-	{ 48, 5240 },
-	{ 50, 5250 },
-	{ 52, 5260 },
-	{ 56, 5280 },
-	{ 58, 5290 },
-	{ 60, 5300 },
-	{ 64, 5320 },
-	{ 100, 5500 },
-	{ 104, 5520 },
-	{ 108, 5540 },
-	{ 112, 5560 },
-	{ 116, 5580 },
-	{ 120, 5600 },
-	{ 124, 5620 },
-	{ 128, 5640 },
-	{ 132, 5660 },
-	{ 136, 5680 },
-	{ 140, 5700 },
-	{ 149, 5745 },
-	{ 152, 5760 },
-	{ 153, 5765 },
-	{ 157, 5785 },
-	{ 160, 5800 },
-	{ 161, 5805 },
-	{ 165, 5825 },
-	{ 183, 4915 },
-	{ 184, 4920 },
-	{ 185, 4925 },
-	{ 187, 4935 },
-	{ 188, 4945 },
-	{ 192, 4960 },
-	{ 196, 4980 },
-	{ 0, -1 }
-};
-
-static struct cf_pair bg_table[] = {
-	/* B/G band */
-	{ 1, 2412 },
-	{ 2, 2417 },
-	{ 3, 2422 },
-	{ 4, 2427 },
-	{ 5, 2432 },
-	{ 6, 2437 },
-	{ 7, 2442 },
-	{ 8, 2447 },
-	{ 9, 2452 },
-	{ 10, 2457 },
-	{ 11, 2462 },
-	{ 12, 2467 },
-	{ 13, 2472 },
-	{ 14, 2484 },
-	{ 0, -1 }
-};
-
-guint32
-utils_freq_to_channel (guint32 freq)
-{
-	int i = 0;
-
-	while (a_table[i].chan && (a_table[i].freq != freq))
-		i++;
-	if (a_table[i].chan)
-		return a_table[i].chan;
-
-	i = 0;
-	while (bg_table[i].chan && (bg_table[i].freq != freq))
-		i++;
-	return bg_table[i].chan;
-}
-
-guint32
-utils_channel_to_freq (guint32 channel, char *band)
-{
-	int i = 0;
-
-	if (!strcmp (band, "a")) {
-		while (a_table[i].chan && (a_table[i].chan != channel))
-			i++;
-		return a_table[i].freq;
-	} else if (!strcmp (band, "bg")) {
-		while (bg_table[i].chan && (bg_table[i].chan != channel))
-			i++;
-		return bg_table[i].freq;
-	}
-
-	return 0;
-}
-
-guint32
-utils_find_next_channel (guint32 channel, int direction, char *band)
-{
-	size_t a_size = sizeof (a_table) / sizeof (struct cf_pair);
-	size_t bg_size = sizeof (bg_table) / sizeof (struct cf_pair);
-	struct cf_pair *pair = NULL;
-
-	if (!strcmp (band, "a")) {
-		if (channel < a_table[0].chan)
-			return a_table[0].chan;
-		if (channel > a_table[a_size - 2].chan)
-			return a_table[a_size - 2].chan;
-		pair = &a_table[0];
-	} else if (!strcmp (band, "bg")) {
-		if (channel < bg_table[0].chan)
-			return bg_table[0].chan;
-		if (channel > bg_table[bg_size - 2].chan)
-			return bg_table[bg_size - 2].chan;
-		pair = &bg_table[0];
-	} else {
-		g_assert_not_reached ();
-		return 0;
-	}
-
-	while (pair->chan) {
-		if (channel == pair->chan)
-			return channel;
-		if ((channel < (pair+1)->chan) && (channel > pair->chan)) {
-			if (direction > 0)	
-				return (pair+1)->chan;
-			else
-				return pair->chan;
-		}
-		pair++;
-	}
-	return 0;
-}
-
 /*
  * utils_ether_addr_valid
  *
@@ -529,7 +242,7 @@ utils_check_ap_compatible (NMAccessPoint *ap,
 	const char *setting_mode;
 	const char *setting_band;
 	NM80211Mode ap_mode;
-	guint32 freq;
+	guint32 freq, channel;
 
 	g_return_val_if_fail (NM_IS_ACCESS_POINT (ap), FALSE);
 	g_return_val_if_fail (NM_IS_CONNECTION (connection), FALSE);
@@ -544,8 +257,10 @@ utils_check_ap_compatible (NMAccessPoint *ap,
 	setting_bssid = nm_setting_wireless_get_bssid (s_wireless);
 	if (setting_bssid) {
 		struct ether_addr ap_addr;
+		const char *hw_addr;
 
-		if (ether_aton_r (nm_access_point_get_hw_address (ap), &ap_addr)) {
+		hw_addr = nm_access_point_get_hw_address (ap);
+		if (hw_addr && ether_aton_r (hw_addr, &ap_addr)) {
 			if (memcmp (setting_bssid->data, &ap_addr, ETH_ALEN))
 				return FALSE;
 		}
@@ -566,25 +281,31 @@ utils_check_ap_compatible (NMAccessPoint *ap,
 	setting_band = nm_setting_wireless_get_band (s_wireless);
 	if (setting_band) {
 		if (!strcmp (setting_band, "a")) {
-			if (freq < 5170 || freq > 5825)
+			if (freq < 4915 || freq > 5825)
 				return FALSE;
 		} else if (!strcmp (setting_band, "bg")) {
-			if (freq < 2412 || freq > 2472)
+			if (freq < 2412 || freq > 2484)
 				return FALSE;
 		}
 	}
 
-	// FIXME: channel check
+	channel = nm_setting_wireless_get_channel (s_wireless);
+	if (channel) {
+		guint32 ap_chan = nm_utils_wifi_freq_to_channel (freq);
+
+		if (channel != ap_chan)
+			return FALSE;
+	}
 
 	s_wireless_sec = (NMSettingWirelessSecurity *) nm_connection_get_setting (connection,
-															    NM_TYPE_SETTING_WIRELESS_SECURITY);
+	                                                                          NM_TYPE_SETTING_WIRELESS_SECURITY);
 
 	return nm_setting_wireless_ap_security_compatible (s_wireless,
-											 s_wireless_sec,
-											 nm_access_point_get_flags (ap),
-											 nm_access_point_get_wpa_flags (ap),
-											 nm_access_point_get_rsn_flags (ap),
-											 nm_access_point_get_mode (ap));
+	                                                   s_wireless_sec,
+	                                                   nm_access_point_get_flags (ap),
+	                                                   nm_access_point_get_wpa_flags (ap),
+	                                                   nm_access_point_get_rsn_flags (ap),
+	                                                   nm_access_point_get_mode (ap));
 }
 
 static gboolean
@@ -618,7 +339,7 @@ connection_valid_for_wired (NMConnection *connection,
 		if (!setting_mac)
 			return TRUE;
 
-		str_mac = nm_device_ethernet_get_hw_address (ethdev);
+		str_mac = nm_device_ethernet_get_permanent_hw_address (ethdev);
 		g_return_val_if_fail (str_mac != NULL, FALSE);
 
 		bin_mac = ether_aton (str_mac);
@@ -657,7 +378,7 @@ connection_valid_for_wireless (NMConnection *connection,
 		const char *str_mac;
 		struct ether_addr *bin_mac;
 
-		str_mac = nm_device_wifi_get_hw_address (wdev);
+		str_mac = nm_device_wifi_get_permanent_hw_address (wdev);
 		g_return_val_if_fail (str_mac != NULL, FALSE);
 
 		bin_mac = ether_aton (str_mac);
@@ -757,6 +478,106 @@ connection_valid_for_cdma (NMConnection *connection,
 	return TRUE;
 }
 
+static guint32
+get_connection_bt_type (NMConnection *connection)
+{
+	NMSettingBluetooth *s_bt;
+	const char *bt_type;
+
+	s_bt = (NMSettingBluetooth *) nm_connection_get_setting (connection, NM_TYPE_SETTING_BLUETOOTH);
+	if (!s_bt)
+		return NM_BT_CAPABILITY_NONE;
+
+	bt_type = nm_setting_bluetooth_get_connection_type (s_bt);
+	g_assert (bt_type);
+
+	if (!strcmp (bt_type, NM_SETTING_BLUETOOTH_TYPE_DUN))
+		return NM_BT_CAPABILITY_DUN;
+	else if (!strcmp (bt_type, NM_SETTING_BLUETOOTH_TYPE_PANU))
+		return NM_BT_CAPABILITY_NAP;
+
+	return NM_BT_CAPABILITY_NONE;
+}
+
+static gboolean
+connection_valid_for_bt (NMConnection *connection,
+                         NMSettingConnection *s_con,
+                         NMDevice *device,
+                         gpointer specific_object)
+{
+	NMSettingBluetooth *s_bt;
+	const GByteArray *array;
+	char *str;
+	const char *hw_addr;
+	int addr_match = FALSE;
+	guint32 bt_type;
+
+	if (strcmp (nm_setting_connection_get_connection_type (s_con), NM_SETTING_BLUETOOTH_SETTING_NAME))
+		return FALSE;
+
+	s_bt = NM_SETTING_BLUETOOTH (nm_connection_get_setting (connection, NM_TYPE_SETTING_BLUETOOTH));
+	if (!s_bt)
+		return FALSE;
+
+	array = nm_setting_bluetooth_get_bdaddr (s_bt);
+	if (!array || (array->len != ETH_ALEN))
+		return FALSE;
+
+	bt_type = get_connection_bt_type (connection);
+	if (!(bt_type & nm_device_bt_get_capabilities (NM_DEVICE_BT (device))))
+		return FALSE;
+
+	str = g_strdup_printf ("%02X:%02X:%02X:%02X:%02X:%02X",
+	                       array->data[0], array->data[1], array->data[2],
+	                       array->data[3], array->data[4], array->data[5]);
+	hw_addr = nm_device_bt_get_hw_address (NM_DEVICE_BT (device));
+	if (hw_addr)
+		addr_match = !g_ascii_strcasecmp (hw_addr, str);
+	g_free (str);
+
+	return addr_match;
+}
+
+static gboolean
+connection_valid_for_wimax (NMConnection *connection,
+                            NMSettingConnection *s_con,
+                            NMDevice *device,
+                            gpointer specific_object)
+{
+	NMDeviceWimax *wimax = NM_DEVICE_WIMAX (device);
+	NMSettingWimax *s_wimax;
+	const char *str_mac;
+	struct ether_addr *bin_mac;
+	const char *connection_type;
+	const GByteArray *setting_mac;
+
+	connection_type = nm_setting_connection_get_connection_type (s_con);
+	if (strcmp (connection_type, NM_SETTING_WIMAX_SETTING_NAME))
+		return FALSE;
+
+	s_wimax = (NMSettingWimax *) nm_connection_get_setting (connection, NM_TYPE_SETTING_WIMAX);
+	if (!s_wimax)
+		return FALSE;
+
+	if (s_wimax) {
+		/* Match MAC address */
+		setting_mac = nm_setting_wimax_get_mac_address (s_wimax);
+		if (!setting_mac)
+			return TRUE;
+
+		str_mac = nm_device_wimax_get_hw_address (wimax);
+		g_return_val_if_fail (str_mac != NULL, FALSE);
+
+		bin_mac = ether_aton (str_mac);
+		g_return_val_if_fail (bin_mac != NULL, FALSE);
+
+		if (memcmp (bin_mac->ether_addr_octet, setting_mac->data, ETH_ALEN))
+			return FALSE;
+	}
+
+	return TRUE;
+}
+
 gboolean
 utils_connection_valid_for_device (NMConnection *connection,
                                    NMDevice *device,
@@ -775,10 +596,20 @@ utils_connection_valid_for_device (NMConnection *connection,
 		return connection_valid_for_wired (connection, s_con, device, specific_object);
 	else if (NM_IS_DEVICE_WIFI (device))
 		return connection_valid_for_wireless (connection, s_con, device, specific_object);
-	else if (NM_IS_GSM_DEVICE (device))
-		return connection_valid_for_gsm (connection, s_con, device, specific_object);
-	else if (NM_IS_CDMA_DEVICE (device))
-		return connection_valid_for_cdma (connection, s_con, device, specific_object);
+	else if (NM_IS_DEVICE_MODEM (device)) {
+		NMDeviceModemCapabilities caps;
+
+		caps = nm_device_modem_get_current_capabilities (NM_DEVICE_MODEM (device));
+		if (caps & NM_DEVICE_MODEM_CAPABILITY_GSM_UMTS)
+			return connection_valid_for_gsm (connection, s_con, device, specific_object);
+		else if (caps & NM_DEVICE_MODEM_CAPABILITY_CDMA_EVDO)
+			return connection_valid_for_cdma (connection, s_con, device, specific_object);
+		else
+			g_warning ("Unhandled modem capabilities 0x%X", caps);
+	} else if (NM_IS_DEVICE_BT (device))
+		return connection_valid_for_bt (connection, s_con, device, specific_object);
+	else if (NM_IS_DEVICE_WIMAX (device))
+		return connection_valid_for_wimax (connection, s_con, device, specific_object);
 	else
 		g_warning ("Unknown device type '%s'", g_type_name (G_OBJECT_TYPE(device)));
 
@@ -801,96 +632,46 @@ utils_filter_connections_for_device (NMDevice *device, GSList *connections)
 	return filtered;
 }
 
-gboolean
-utils_mac_valid (const struct ether_addr *addr)
-{
-	guint8 invalid1[ETH_ALEN] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};
-	guint8 invalid2[ETH_ALEN] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
-	guint8 invalid3[ETH_ALEN] = {0x44, 0x44, 0x44, 0x44, 0x44, 0x44};
-	guint8 invalid4[ETH_ALEN] = {0x00, 0x30, 0xb4, 0x00, 0x00, 0x00}; /* prism54 dummy MAC */
-
-	g_return_val_if_fail (addr != NULL, FALSE);
-
-	/* Compare the AP address the card has with invalid ethernet MAC addresses. */
-	if (!memcmp (addr->ether_addr_octet, &invalid1, ETH_ALEN))
-		return FALSE;
-
-	if (!memcmp (addr->ether_addr_octet, &invalid2, ETH_ALEN))
-		return FALSE;
-
-	if (!memcmp (addr->ether_addr_octet, &invalid3, ETH_ALEN))
-		return FALSE;
-
-	if (!memcmp (addr->ether_addr_octet, &invalid4, ETH_ALEN))
-		return FALSE;
-
-	if (addr->ether_addr_octet[0] & 1) /* Multicast addresses */
-		return FALSE;
-	
-	return TRUE;
-}
-
 char *
-utils_ether_ntop (const struct ether_addr *mac)
+utils_hash_ap (const GByteArray *ssid,
+               NM80211Mode mode,
+               guint32 flags,
+               guint32 wpa_flags,
+               guint32 rsn_flags)
 {
-	/* we like leading zeros and all-caps, instead
-	 * of what glibc's ether_ntop() gives us
-	 */
-	return g_strdup_printf ("%02X:%02X:%02X:%02X:%02X:%02X",
-	                        mac->ether_addr_octet[0], mac->ether_addr_octet[1],
-	                        mac->ether_addr_octet[2], mac->ether_addr_octet[3],
-	                        mac->ether_addr_octet[4], mac->ether_addr_octet[5]);
-}
+	unsigned char input[66];
 
+	memset (&input[0], 0, sizeof (input));
 
-static void
-add_one_name (gpointer data, gpointer user_data)
-{
-	NMExportedConnection *exported = NM_EXPORTED_CONNECTION (data);
-	NMConnection *connection;
-	NMSettingConnection *s_con;
-	const char *id;
-	GSList **list = (GSList **) user_data;
+	if (ssid)
+		memcpy (input, ssid->data, ssid->len);
 
-	connection = nm_exported_connection_get_connection (exported);
-	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION));
-	id = nm_setting_connection_get_id (s_con);
-	g_assert (id);
-	*list = g_slist_append (*list, (gpointer) id);
-}
+	if (mode == NM_802_11_MODE_INFRA)
+		input[32] |= (1 << 0);
+	else if (mode == NM_802_11_MODE_ADHOC)
+		input[32] |= (1 << 1);
+	else
+		input[32] |= (1 << 2);
 
-char *
-utils_next_available_name (GSList *connections, const char *format)
-{
-	GSList *names = NULL, *iter;
-	char *cname = NULL;
-	int i = 0;
+	/* Separate out no encryption, WEP-only, and WPA-capable */
+	if (  !(flags & NM_802_11_AP_FLAGS_PRIVACY)
+	    && (wpa_flags == NM_802_11_AP_SEC_NONE)
+	    && (rsn_flags == NM_802_11_AP_SEC_NONE))
+		input[32] |= (1 << 3);
+	else if (   (flags & NM_802_11_AP_FLAGS_PRIVACY)
+	         && (wpa_flags == NM_802_11_AP_SEC_NONE)
+	         && (rsn_flags == NM_802_11_AP_SEC_NONE))
+		input[32] |= (1 << 4);
+	else if (   !(flags & NM_802_11_AP_FLAGS_PRIVACY)
+	         &&  (wpa_flags != NM_802_11_AP_SEC_NONE)
+	         &&  (rsn_flags != NM_802_11_AP_SEC_NONE))
+		input[32] |= (1 << 5);
+	else
+		input[32] |= (1 << 6);
 
-	g_slist_foreach (connections, add_one_name, &names);
-
-	if (g_slist_length (names) == 0)
-		return g_strdup_printf (format, 1);
-
-	/* Find the next available unique connection name */
-	while (!cname && (i++ < 10000)) {
-		char *temp;
-		gboolean found = FALSE;
-
-		temp = g_strdup_printf (format, i);
-		for (iter = names; iter; iter = g_slist_next (iter)) {
-			if (!strcmp (iter->data, temp)) {
-				found = TRUE;
-				break;
-			}
-		}
-		if (!found)
-			cname = temp;
-		else
-			g_free (temp);
-	}
-
-	g_slist_free (names);
-	return cname;
+	/* duplicate it */
+	memcpy (&input[33], &input[0], 32);
+	return g_compute_checksum_for_data (G_CHECKSUM_MD5, input, sizeof (input));
 }
 
 typedef struct {
@@ -946,3 +727,46 @@ utils_escape_notify_message (const char *src)
 	return g_string_free (escaped, FALSE);
 }
 
+GnomeKeyringAttributeList *
+utils_create_keyring_add_attr_list (NMConnection *connection,
+                                    const char *connection_uuid,
+                                    const char *connection_id,
+                                    const char *setting_name,
+                                    const char *setting_key,
+                                    char **out_display_name)
+{
+	GnomeKeyringAttributeList *attrs = NULL;
+	NMSettingConnection *s_con;
+
+	if (connection) {
+		s_con = (NMSettingConnection *) nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION);
+		g_return_val_if_fail (s_con != NULL, NULL);
+		connection_uuid = nm_setting_connection_get_uuid (s_con);
+		connection_id = nm_setting_connection_get_id (s_con);
+	}
+
+	g_return_val_if_fail (connection_uuid != NULL, NULL);
+	g_return_val_if_fail (connection_id != NULL, NULL);
+	g_return_val_if_fail (setting_name != NULL, NULL);
+	g_return_val_if_fail (setting_key != NULL, NULL);
+
+	if (out_display_name) {
+		*out_display_name = g_strdup_printf ("Network secret for %s/%s/%s",
+		                                     connection_id,
+		                                     setting_name,
+		                                     setting_key);
+	}
+
+	attrs = gnome_keyring_attribute_list_new ();
+	gnome_keyring_attribute_list_append_string (attrs,
+	                                            KEYRING_UUID_TAG,
+	                                            connection_uuid);
+	gnome_keyring_attribute_list_append_string (attrs,
+	                                            KEYRING_SN_TAG,
+	                                            setting_name);
+	gnome_keyring_attribute_list_append_string (attrs,
+	                                            KEYRING_SK_TAG,
+	                                            setting_key);
+	return attrs;
+}
+
