@@ -20,6 +20,8 @@
  * (C) Copyright 2007 - 2010 Red Hat, Inc.
  */
 
+#include "config.h"
+
 #include <glib/gi18n.h>
 #include <ctype.h>
 #include <string.h>
@@ -29,7 +31,6 @@
 
 #include "eap-method.h"
 #include "wireless-security.h"
-#include "gconf-helpers.h"
 
 #define I_NAME_COLUMN   0
 #define I_METHOD_COLUMN 1
@@ -118,7 +119,6 @@ add_to_size_group (EAPMethod *parent, GtkSizeGroup *group)
 static void
 fill_connection (EAPMethod *parent, NMConnection *connection)
 {
-	NMSettingConnection *s_con;
 	NMSetting8021x *s_8021x;
 	NMSetting8021xCKFormat format = NM_SETTING_802_1X_CK_FORMAT_UNKNOWN;
 	GtkWidget *widget;
@@ -130,9 +130,6 @@ fill_connection (EAPMethod *parent, NMConnection *connection)
 	int peapver_active = 0;
 	GError *error = NULL;
 
-	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION));
-	g_assert (s_con);
-
 	s_8021x = NM_SETTING_802_1X (nm_connection_get_setting (connection, NM_TYPE_SETTING_802_1X));
 	g_assert (s_8021x);
 
@@ -152,10 +149,6 @@ fill_connection (EAPMethod *parent, NMConnection *connection)
 		g_clear_error (&error);
 	}
 
-	nm_gconf_set_ignore_ca_cert (nm_setting_connection_get_uuid (s_con),
-	                             FALSE,
-	                             eap_method_get_ignore_ca_cert (parent));
-
 	widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_peap_version_combo"));
 	peapver_active = gtk_combo_box_get_active (GTK_COMBO_BOX (widget));
 	switch (peapver_active) {
@@ -220,8 +213,7 @@ inner_auth_combo_changed_cb (GtkWidget *combo, gpointer user_data)
 static GtkWidget *
 inner_auth_combo_init (EAPMethodPEAP *method,
                        NMConnection *connection,
-                       NMSetting8021x *s_8021x,
-                       gboolean secrets_only)
+                       NMSetting8021x *s_8021x)
 {
 	EAPMethod *parent = (EAPMethod *) method;
 	GtkWidget *combo;
@@ -246,8 +238,7 @@ inner_auth_combo_init (EAPMethodPEAP *method,
 	                                      connection,
 	                                      EAP_METHOD_SIMPLE_TYPE_MSCHAP_V2,
 	                                      TRUE,
-	                                      method->is_editor,
-	                                      secrets_only);
+	                                      method->is_editor);
 	gtk_list_store_append (auth_model, &iter);
 	gtk_list_store_set (auth_model, &iter,
 	                    I_NAME_COLUMN, _("MSCHAPv2"),
@@ -263,8 +254,7 @@ inner_auth_combo_init (EAPMethodPEAP *method,
 	                                connection,
 	                                EAP_METHOD_SIMPLE_TYPE_MD5,
 	                                TRUE,
-	                                method->is_editor,
-	                                secrets_only);
+	                                method->is_editor);
 	gtk_list_store_append (auth_model, &iter);
 	gtk_list_store_set (auth_model, &iter,
 	                    I_NAME_COLUMN, _("MD5"),
@@ -280,8 +270,7 @@ inner_auth_combo_init (EAPMethodPEAP *method,
 	                                connection,
 	                                EAP_METHOD_SIMPLE_TYPE_GTC,
 	                                TRUE,
-	                                method->is_editor,
-	                                secrets_only);
+	                                method->is_editor);
 	gtk_list_store_append (auth_model, &iter);
 	gtk_list_store_set (auth_model, &iter,
 	                    I_NAME_COLUMN, _("GTC"),
@@ -318,8 +307,7 @@ update_secrets (EAPMethod *parent, NMConnection *connection)
 EAPMethodPEAP *
 eap_method_peap_new (WirelessSecurity *ws_parent,
                      NMConnection *connection,
-                     gboolean is_editor,
-                     gboolean secrets_only)
+                     gboolean is_editor)
 {
 	EAPMethod *parent;
 	EAPMethodPEAP *method;
@@ -336,14 +324,12 @@ eap_method_peap_new (WirelessSecurity *ws_parent,
 	                          destroy,
 	                          UIDIR "/eap-method-peap.ui",
 	                          "eap_peap_notebook",
-	                          "eap_peap_anon_identity_entry");
+	                          "eap_peap_anon_identity_entry",
+	                          FALSE);
 	if (!parent)
 		return NULL;
 
-	eap_method_nag_init (parent,
-	                     "eap_peap_ca_cert_button",
-	                     connection,
-	                     FALSE);
+	eap_method_nag_init (parent, "eap_peap_ca_cert_button", connection);
 
 	method = (EAPMethodPEAP *) parent;
 	method->sec_parent = ws_parent;
@@ -370,7 +356,7 @@ eap_method_peap_new (WirelessSecurity *ws_parent,
 		}
 	}
 
-	widget = inner_auth_combo_init (method, connection, s_8021x, secrets_only);
+	widget = inner_auth_combo_init (method, connection, s_8021x);
 	inner_auth_combo_changed_cb (widget, (gpointer) method);
 
 	widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_peap_version_combo"));
@@ -399,25 +385,6 @@ eap_method_peap_new (WirelessSecurity *ws_parent,
 	                  (GCallback) wireless_security_changed_cb,
 	                  ws_parent);
 
-	if (secrets_only) {
-		widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_peap_anon_identity_label"));
-		gtk_widget_hide (widget);
-		widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_peap_anon_identity_entry"));
-		gtk_widget_hide (widget);
-		widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_peap_ca_cert_label"));
-		gtk_widget_hide (widget);
-		widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_peap_ca_cert_button"));
-		gtk_widget_hide (widget);
-		widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_peap_inner_auth_label"));
-		gtk_widget_hide (widget);
-		widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_peap_inner_auth_combo"));
-		gtk_widget_hide (widget);
-		widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_peap_version_label"));
-		gtk_widget_hide (widget);
-		widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_peap_version_combo"));
-		gtk_widget_hide (widget);
-	}
-
 	return method;
 }
 
